@@ -50,9 +50,9 @@ def normalize_adjustment(feature_values):
 
 def get_sample_times(bucket,measurment_occaisons,feature_values,abnormal_ratio,measure_max,sampling_function=None):
     if bucket == "equal":
-        return(np.linspace(0,0.99999,measurment_occaisons))
+        return(np.linspace(0,((1-abnormal_ratio)+abnormal_ratio*(measurment_occaisons/measure_max)),measurment_occaisons))
     elif bucket == "random":
-        return(np.random.uniform(low=0.0, high=1.0, size=measurment_occaisons))
+        return(np.random.uniform(low=0.0, high=((1-abnormal_ratio)+abnormal_ratio*(measurment_occaisons/measure_max)), size=measurment_occaisons))
     elif bucket == "not-random":
         abnormal_x={}
         either=[]
@@ -77,9 +77,9 @@ def get_sample_times(bucket,measurment_occaisons,feature_values,abnormal_ratio,m
         neither=np.insert(neither,0,False)[0:-1]
         return(np.cumsum(step_size*neither+np.minimum.reduce([abnormal_x[x] for x in abnormal_x])))
     elif bucket == "custom-no-features":
-        return(sampling_function(measurment_occaisons))
+        return(sampling_function(measurment_occaisons,abnormal_ratio,measure_max))
     elif bucket == "custom-feature-values":
-        return(sampling_function(measurment_occaisons,feature_values))
+        return(sampling_function(measurment_occaisons,feature_values,abnormal_ratio,measure_max))
 
 
 def split_between_cutpoints(times,entity_to_split,stationarity_change_points):
@@ -385,17 +385,22 @@ class long_data_set:
         if len(self.random_effects) > 0:
             if "intercept" in self.b_dict:
                 cut_off=np.quantile(b_df["intercept"],self.percentile_sort_cutoff)
-                sub_value=np.quantile(b_df["intercept"],self.percentile_sub)
+                sub_value=np.quantile(b_df["intercept"].abs(),self.percentile_sub)
                 b_df['sort_col']=b_df["intercept"].apply(lambda v: data_sorter(v,cut_off,sub_value))
             elif "time" in self.b_dict:
                 cut_off=np.quantile(b_df["time"],self.percentile_sort_cutoff)
+                sub_value=np.quantile(b_df["time"].abs(),self.percentile_sub)
                 b_df['sort_col']=b_df["time"].apply(lambda v: data_sorter(v,cut_off,sub_value))
             elif "trend-time" in self.b_dict:
                 cut_off=np.quantile(b_df["trend-time"],self.percentile_sort_cutoff)
+                sub_value=np.quantile(b_df["trend-time"].abs(),self.percentile_sub)
                 b_df['sort_col']=b_df["trend-time"].apply(lambda v: data_sorter(v,cut_off,sub_value))
             else:
                 sort_col=b_df.columns.values[0]
-                b_df['sort_col']=b_df[sort_col].abs()
+                cut_off=np.quantile(sort_col,self.percentile_sort_cutoff)
+                b_df['sort_col']=b_df[sort_col]
+                sub_value=np.quantile(b_df['sort_col'].abs(),self.percentile_sub)
+                b_df['sort_col']=b_df["sort_col"].apply(lambda v: data_sorter(v,cut_off,sub_value))
             b_df=b_df.sort_values(by=['sort_col'])
             b_df=b_df.reset_index(drop=True)
         for effect in self.random_effects:
